''' '''

import logging

logger = logging.getLogger(__name__)

from .base import *  # noqa: F401, F402, E402, F403
from .cam import *  # noqa: F401, F402, E402, F403
from .detectors import *  # noqa: F401, F402, E402, F403

# NOTE: the following imports are here for backward compatibility with
# previous ophyd versions. This does not represent all available plugins
# in ophyd. For that, import directly from ophyd.areadetector.plugins.
from .plugins import (ColorConvPlugin, FilePlugin, HDF5Plugin, ImagePlugin,  # noqa: F401, F402, E402
                      JPEGPlugin, MagickPlugin, NetCDFPlugin, NexusPlugin,  # noqa: F401
                      OverlayPlugin, ProcessPlugin, ROIPlugin, StatsPlugin,
                      TIFFPlugin, TransformPlugin, get_areadetector_plugin,
                      plugin_from_pvname, register_plugin)

from .common_plugins import *  # noqa: F401, F402, E402, F403
from .trigger_mixins import *  # noqa: F401, F402, E402, F403
from .paths import EpicsPathSignal  # noqa: F401, F402, E402, F403



"""
Documentation helpers that can be used in packages relying on ophyd devices.
"""

from . import templates
from .utils import (autodoc_default_options, autosummary_context,
                    get_device_info, intersphinx_mapping, setup)

__all__ = [
    'autodoc_default_options',
    'autosummary_context',
    'get_device_info',
    'intersphinx_mapping',
    'setup',
    'templates',
]



import pytest

from ophyd.docs import get_device_info


def test_get_device_info_scaler():
    # `get_device_info` done by name, due to how sphinx directives work:
    info = get_device_info('ophyd.scaler', 'EpicsScaler')
    assert len(info)

    # A list of component information dictionaries:
    components_by_attr = {item['attr']: item for item in info}
    assert 'count_mode' in components_by_attr
    assert 'channels (DDC)' in components_by_attr

    # And detailed nested component information for a DDC:
    nested = components_by_attr['channels (DDC)']['nested_components']
    nested_components_by_attr = {item['attr']: item for item in nested}
    assert 'chan1' in nested_components_by_attr


@pytest.mark.parametrize(
    'module, class_name',
    [
        pytest.param('ophyd.areadetector.base', 'ADBase'),
        pytest.param('ophyd.areadetector.cam', 'PcoDetectorIO'),
        pytest.param('ophyd.areadetector.cam', 'PcoDetectorSimIO'),
        pytest.param('ophyd.areadetector.common_plugins', 'CommonAttributePlugin'),
        pytest.param('ophyd.areadetector.common_plugins', 'CommonGatherPlugin'),
        pytest.param('ophyd.areadetector.common_plugins', 'CommonOverlayPlugin'),
        pytest.param('ophyd.areadetector.common_plugins', 'CommonPlugins'),
        pytest.param('ophyd.areadetector.common_plugins', 'CommonROIStatPlugin'),
        pytest.param('ophyd.areadetector.detectors', 'DetectorBase'),
        pytest.param('ophyd.areadetector.plugins', 'AttrPlotPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'AttributeNPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'AttributePlugin'),
        pytest.param('ophyd.areadetector.plugins', 'CircularBuffPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'CodecPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'FFTPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'GatherNPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'Overlay'),
        pytest.param('ophyd.areadetector.plugins', 'PluginBase'),
        pytest.param('ophyd.areadetector.plugins', 'PosPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'PvaPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'ROIStatNPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'ROIStatPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'ScatterPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'TimeSeriesNPlugin'),
        pytest.param('ophyd.areadetector.plugins', 'TimeSeriesPlugin'),
        pytest.param('ophyd.epics_motor', 'EpicsMotor'),
        pytest.param('ophyd.epics_motor', 'MotorBundle'),
        pytest.param('ophyd.flyers', 'AreaDetectorTimeseriesCollector'),
        pytest.param('ophyd.flyers', 'WaveformCollector'),
        pytest.param('ophyd.mca', 'EpicsDXP'),
        pytest.param('ophyd.mca', 'EpicsDXPBaseSystem'),
        pytest.param('ophyd.mca', 'EpicsDXPLowLevel'),
        pytest.param('ophyd.mca', 'EpicsDXPLowLevelParameter'),
        pytest.param('ophyd.mca', 'EpicsDXPMapping'),
        pytest.param('ophyd.mca', 'EpicsMCACallback'),
        pytest.param('ophyd.mca', 'EpicsMCARecord'),
        pytest.param('ophyd.mca', 'ROI'),
        pytest.param('ophyd.mca', 'SoftDXPTrigger'),
        pytest.param('ophyd.pseudopos', 'PseudoPositioner'),
        pytest.param('ophyd.pseudopos', 'PseudoSingle'),
        pytest.param('ophyd.pv_positioner', 'PVPositioner'),
        pytest.param('ophyd.quadem', 'QuadEMPort'),
        pytest.param('ophyd.scaler', 'EpicsScaler'),
        pytest.param('ophyd.scaler', 'ScalerCH'),
        pytest.param('ophyd.scaler', 'ScalerChannel'),
        pytest.param('ophyd.sim', 'ABDetector'),
        pytest.param('ophyd.sim', 'DetWithConf'),
        pytest.param('ophyd.sim', 'DetWithCountTime'),
        pytest.param('ophyd.sim', 'DirectImage'),
        pytest.param('ophyd.sim', 'Syn2DGauss'),
        pytest.param('ophyd.sim', 'SynAxis'),
        pytest.param('ophyd.sim', 'SynGauss'),
        pytest.param('ophyd.tests.test_sim', 'Sample'),
        pytest.param('ophyd.tests.test_sim', 'SampleNested'),
    ]
)
def test_get_device_info_smoke(module, class_name):
    get_device_info(module, class_name)
